@@ -20,7 +20,7 @@
     
     License
     -------
-    Copyright 2008 - 2010 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2008 - 2011 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -45,8 +45,10 @@ except ImportError:
     import simplejson as json
 
 import gettext
+import shutil
 import logging
 import os
+import subprocess
 import sys
 import tarfile
 import urllib2
@@ -61,8 +63,6 @@ _ = gettext.gettext
 _presets = {}
 _log = logging.getLogger("arista.presets")
 
-UPDATE_LOCATION = "http://www.transcoder.org/media/presets/"
-
 class Fraction(gst.Fraction):
     """
         An object for storing a fraction as two integers. This is a subclass
@@ -101,7 +101,7 @@ class Author(object):
         self.email = email
     
     def __repr__(self):
-        return "%s <%s>" % (self.name, self.email)
+        return (self.name or self.email) and "%s <%s>" % (self.name, self.email) or ""
 
 class Device(object):
     """
@@ -165,6 +165,13 @@ class Device(object):
         else:
             return "%s %s" % (self.make, self.model)
     
+    @property
+    def short_name(self):
+        """
+            Return the short name of this device preset.
+        """
+        return ".".join(os.path.basename(self.filename).split(".")[:-1])
+    
     @property
     def default_preset(self):
         """
@@ -176,10 +183,10 @@ class Device(object):
             @return: The default preset for this device
             @raise ValueError: No presets have been defined for this device
         """
-        if self.default:
+        if self.default in self.presets:
             preset = self.presets[self.default]
         elif len(self.presets):
-            preset = self.presets[0]
+            preset = self.presets.values()[0]
         else:
             raise ValueError(_("No presets have been defined for " \
                                  "%(name)s") % { "name": self.name })
@@ -206,7 +213,7 @@ class Device(object):
             rates = []
             for x in preset.acodec.rate[0], preset.acodec.rate[1], preset.vcodec.rate[0], preset.vcodec.rate[1]:
                 if isinstance(x, gst.Fraction):
-                    if x.num == x.denom:
+                    if x.denom == 1:
                         rates.append("%s" % x.num)
                     else:
                         rates.append("%s/%s" % (x.num, x.denom))
@@ -215,8 +222,15 @@ class Device(object):
         
             data["presets"].append({
                 "name": preset.name,
+                "description": preset.description,
+                "author": {
+                    "name": preset.author.name,
+                    "email": preset.author.email,
+                },
                 "container": preset.container,
                 "extension": preset.extension,
+                "icon": preset.icon,
+                "version": preset.version,
                 "acodec": {
                     "name": preset.acodec.name,
                     "container": preset.acodec.container,
@@ -233,11 +247,41 @@ class Device(object):
                     "passes": preset.vcodec.passes,
                     "width": preset.vcodec.width,
                     "height": preset.vcodec.height,
+                    "transform": preset.vcodec.transform,
                 },
             })
         
         return json.dumps(data, indent=4)
-
+    
+    def save(self):
+        """
+            Save this device and its presets to a file. The device.filename must
+            be set to a valid path or an error will be thrown.
+        """
+        open(self.filename, "w").write(self.json)
+   
+    def export(self, filename):
+        """
+            Export this device and all presets to a file. Creates a bzipped
+            tarball of the JSON and all associated images that can be easily
+            imported later.
+        """
+        # Make sure all changes are saved
+        self.save()
+        
+        # Gather image files
+        images = set()
+        for name, preset in self.presets.items():
+            if preset.icon:
+                images.add(preset.icon[7:])
+        
+        files = " ".join([os.path.basename(self.filename)] + list(images))
+        
+        cwd = os.getcwd()
+        os.chdir(os.path.dirname(self.filename))
+        subprocess.call("tar -cjf %s %s" % (filename, files), shell=True)
+        os.chdir(cwd)
+    
     @staticmethod
     def from_json(data):
         parsed = json.loads(data)
@@ -260,13 +304,19 @@ class Device(object):
             vcodec = preset.get("vcodec", {})
             device.presets[preset.get("name", "")] = Preset(**{
                 "name": preset.get("name", ""),
+                "description": preset.get("description", device.description),
+                "author": Author(
+                    name = preset.get("author", {}).get("name", device.author.name),
+                    email = preset.get("author", {}).get("email", device.author.email),
+                ),
                 "container": preset.get("container", ""),
                 "extension": preset.get("extension", ""),
-                "icon": preset.get("icon", ""),
+                "version": preset.get("version", device.version),
+                "icon": preset.get("icon", device.icon),
                 "acodec": AudioCodec(**{
                     "name": acodec.get("name", ""),
                     "container": acodec.get("container", ""),
-                    "rate": acodec.get("rate", []),
+                    "rate": [int(x) for x in acodec.get("rate", [])],
                     "passes": acodec.get("passes", []),
                     "width": acodec.get("width", []),
                     "depth": acodec.get("depth", []),
@@ -279,6 +329,7 @@ class Device(object):
                     "passes": vcodec.get("passes", []),
                     "width": vcodec.get("width", []),
                     "height": vcodec.get("height", []),
+                    "transform": vcodec.get("transform", ""),
                 }),
                 "device": device,
             })
@@ -291,7 +342,8 @@ class Preset(object):
         device.
     """
     def __init__(self, name = "", container = "", extension = "", 
-                 acodec = None, vcodec = None, device = None, icon = None):
+                 acodec = None, vcodec = None, device = None, icon = None,
+                 version = None, description = None, author = None):
         """
             @type name: str
             @param name: The name of the preset, e.g. "High Quality"
@@ -307,11 +359,14 @@ class Preset(object):
             @param device: A link back to the device this preset belongs to
         """
         self.name = name
+        self.description = description
+        self.author = author
         self.container = container
         self.extension = extension
         self.acodec = acodec
         self.vcodec = vcodec
         self.device = device
+        self.version = version
         self.icon = icon
     
     def __repr__(self):
@@ -325,6 +380,17 @@ class Preset(object):
         """
         return max(len(self.vcodec.passes), len(self.acodec.passes))
     
+    @property
+    def slug(self):
+        """
+            @rtype: str
+            @return: A slug based on the preset name safe to use as a filename
+                     or in links
+        """
+        slug = ".".join(os.path.basename(self.device.filename).split(".")[:-1]) + "-" + self.name.lower()
+        
+        return slug.replace(" ", "_").replace("'", "").replace("/", "")
+    
     def check_elements(self, callback, *args):
         """
             Check the elements used in this preset. If they don't exist then
@@ -426,11 +492,12 @@ class VideoCodec(Codec):
     """
         Settings for encoding video.
     """
-    def __init__(self, name=None, container=None, rate=None, passes=None, width=None, height=None):
+    def __init__(self, name=None, container=None, rate=None, passes=None, width=None, height=None, transform=None):
         Codec.__init__(self, name=name, container=container, passes=passes)
         self.rate = rate and rate or (Fraction("1"), Fraction("60"))
         self.width = width and width or (2, 1920)
         self.height = height and height or (2, 1080)
+        self.transform = transform
 
 def load(filename):
     """
@@ -466,8 +533,8 @@ def load_directory(directory):
         if filename.endswith("json"):
             try:
                 _presets[filename[:-5]] = load(os.path.join(directory, filename))
-            except:
-                _log.warning("Problem loading %s!" % filename)
+            except Exception, e:
+                _log.warning("Problem loading %s! %s" % (filename, str(e)))
     return _presets
 
 def get():
@@ -551,95 +618,31 @@ def fetch(location, name):
     
     return updated
 
-def check_for_updates(location = UPDATE_LOCATION):
-    """
-        Check for updated presets from a central server.
-        
-        @type location: str
-        @param location: The directory where latest.txt and all preset files
-                         can be found on the server
-        @rtype: list
-        @return: A list of [(location, name), (location, name), ...] for each
-                 preset that has an update available
-    """
-    _log.debug(_("Checking for device preset updates..."))
-    
-    updates = []
-    
-    if not location.endswith("/"):
-        location = location + "/"
-    
-    try:
-        f = urllib2.urlopen(location + "latest.txt")
-    except urllib2.URLError:
-        return updates
-    
-    try:
-        for line in f.readlines():
-            if not line.strip():
-                continue
-            
-            parts = [part.strip() for part in line.split(",")]
-            
-            if len(parts) == 2:
-                name, version = parts
-                if _presets.has_key(name):
-                    if _presets[name].version >= version:
-                        _log.debug(_("Device preset %(name)s is up to date") % {
-                            "name": name,
-                        })
-                    else:
-                        _log.debug(_("Found updated device preset %(name)s") % {
-                            "name": name,
-                        })
-                        try:
-                            updates.append((location, name))
-                        except Exception, e:
-                            _log.error(_("Error installing preset %(name)s " \
-                                         "from %(location)s: %(error)s") % {
-                                "name": name,
-                                "location": location,
-                                "error": str(e),
-                            })
-            else:
-                _log.warning(_("Malformed plugin version line %(line)s") % {
-                    "line": line,
-                })
-    except:
-        _log.warning(_("There was a problem accessing %(location)slatest.txt!") % {
-            "location": location,
-        })
-    
-    return updates
-
-def check_and_install_updates(location = UPDATE_LOCATION):
-    """
-        Check for and install updated presets from a central server. This is
-        equivalent to calling install_preset with each tuple returned from
-        check_for_updates.
-        
-        @type location: str
-        @param location: The directory where presets.txt and all preset files
-                         can be found on the server
-    """
-    updates = check_for_updates(location)
-    
-    if updates:
-        for loc, name in updates:
-            fetch(loc, name)
-    else:
-        _log.debug(_("All device presets are up to date!"))
-
-def reset():
-    # Automatically load presets - system, home, current path
+def reset(overwrite=False, ignore_initial=False):
+    # Automatically load presets
     global _presets
     
     _presets = {}
     
-    for path in reversed(utils.get_search_paths()):
-        full = os.path.join(path, "presets")
-        if os.path.exists(full):
-            load_directory(full)
+    load_path = utils.get_write_path("presets")
+    if ignore_initial or not os.path.exists(os.path.join(load_path, ".initial_complete")):
+        # Do initial population of presets from system install / cwd
+        if not os.path.exists(load_path):
+            os.makedirs(load_path)
+            
+        # Write file to say we have done this
+        open(os.path.join(load_path, ".initial_complete"), "w").close()
+        
+        # Copy actual files
+        for path in utils.get_search_paths():
+            full = os.path.join(path, "presets")
+            if full != load_path and os.path.exists(full):
+                for f in os.listdir(full):
+                    # Do not overwrite existing files
+                    if overwrite or not os.path.exists(os.path.join(load_path, f)):
+                        shutil.copy2(os.path.join(full, f), load_path)
+    
+    load_directory(load_path)
 
 reset()
 
