@@ -8,7 +8,7 @@
     
     License
     -------
-    Copyright 2008 - 2010 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2008 - 2011 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -201,6 +201,8 @@ class TranscodeQueue(gobject.GObject):
             
             def error(transcoder, errorstr):
                 self.emit("entry-error", item, errorstr)
+                self._queue.pop(0)
+                self.pipe_running = False
             
             item.transcoder.connect("discovered", discovered)
             item.transcoder.connect("pass-setup", pass_setup)
