@@ -7,7 +7,7 @@
     
     License
     -------
-    Copyright 2009 - 2010 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2009 - 2011 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -85,8 +85,9 @@ class TranscoderOptions(object):
         Options pertaining to the input/output location, presets, 
         subtitles, etc.
     """
-    def __init__(self, uri = None, preset = None, output_uri = None, 
-                 subfile = None, font = "Sans Bold 16", deinterlace = None):
+    def __init__(self, uri = None, preset = None, output_uri = None, ssa = False,
+                 subfile = None, subfile_charset = None, font = "Sans Bold 16",
+                 deinterlace = None, crop = None):
         """
             @type uri: str
             @param uri: The URI to the input file, device, or stream
@@ -96,24 +97,35 @@ class TranscoderOptions(object):
             @param output_uri: The URI to the output file, device, or stream
             @type subfile: str
             @param subfile: The location of the subtitle file
+            @type subfile_charset: str
+            @param subfile_charset: Subtitle file character encoding, e.g.
+                                    'utf-8' or 'latin-1'
             @type font: str
             @param font: Pango font description
             @type deinterlace: bool
             @param deinterlace: Force deinterlacing of the input data
+            @type crop: int tuple
+            @param crop: How much should be cropped on each side
+                                    (top, right, bottom, left)
         """
-        self.reset(uri, preset, output_uri, subfile, font, deinterlace)
+        self.reset(uri, preset, output_uri, ssa,subfile, subfile_charset, font,
+                   deinterlace, crop)
     
-    def reset(self, uri = None, preset = None, output_uri = None,
-              subfile = None, font = "Sans Bold 16", deinterlace = None):
+    def reset(self, uri = None, preset = None, output_uri = None, ssa = False,
+              subfile = None, subfile_charset = None, font = "Sans Bold 16",
+              deinterlace = None, crop = None):
         """
             Reset the input options to nothing.
         """
         self.uri = uri
         self.preset = preset
         self.output_uri = output_uri
+        self.ssa = ssa
         self.subfile = subfile
+        self.subfile_charset = subfile_charset
         self.font = font
         self.deinterlace = deinterlace
+        self.crop = crop
 
 # =============================================================================
 # The Transcoder
@@ -156,7 +168,7 @@ class Transcoder(gobject.GObject):
         if options.uri.startswith("dvd://") and len(options.uri.split("@")) < 2:
             # This is a DVD and no title is yet selected... find the best
             # candidate by searching for the longest title!
-            self.options.uri += "@1"
+            self.options.uri += "@0"
             self.dvd_infos = []
             
             def _got_info(info, is_media):
@@ -276,7 +288,7 @@ class Transcoder(gobject.GObject):
         elif self.infile.startswith("file://"):
             filename = self.infile
         else:
-            filename = "file://" + self.infile
+            filename = "file://" + os.path.abspath(self.infile)
             
         return "uridecodebin uri=\"%s\" name=dmux" % filename
     
@@ -354,42 +366,63 @@ class Transcoder(gobject.GObject):
                             setattr(self.preset.vcodec, field, cur)
             
             # =================================================================
-            # Calculate video width/height and add black bars if necessary
+            # Calculate video width/height, crop and add black bars if necessary
             # =================================================================
+            vcrop = ""
+            crop = [0, 0, 0, 0]
+            if self.options.crop:
+                crop = self.options.crop
+                vcrop = "videocrop top=%i right=%i bottom=%i left=%i ! "  % \
+                        (crop[0], crop[1], crop[2], crop[3])
+            
             wmin, wmax = self.preset.vcodec.width
             hmin, hmax = self.preset.vcodec.height
             
-            owidth, oheight = self.info.videowidth, self.info.videoheight
+            owidth = self.info.videowidth - crop[1] - crop[3]
+            oheight = self.info.videoheight - crop[0] - crop[2]
+            
+            try:
+                if self.info.videocaps[0].has_key("pixel-aspect-ratio"):
+                    owidth = int(owidth * float(self.info.videocaps[0]["pixel-aspect-ratio"]))
+            except KeyError:
+                # The videocaps we are looking for may not even exist, just ignore
+                pass
+            
             width, height = owidth, oheight
             
-            # Scale width / height down
-            if owidth > wmax:
+            # Scale width / height to fit requested min/max
+            if owidth < wmin:
+                width = wmin
+                height = int((float(wmin) / owidth) * oheight)
+            elif owidth > wmax:
                 width = wmax
                 height = int((float(wmax) / owidth) * oheight)
-            if height > hmax:
+            
+            if height < hmin:
+                height = hmin
+                width = int((float(hmin) / oheight) * owidth)
+            elif height > hmax:
                 height = hmax
                 width = int((float(hmax) / oheight) * owidth)
             
             # Add any required padding
+            # TODO: Remove the extra colorspace conversion when no longer
+            #       needed, but currently xvidenc and possibly others will fail
+            #       without it!
             vbox = ""
             if width < wmin and height < hmin:
                 wpx = (wmin - width) / 2
                 hpx = (hmin - height) / 2
-                vbox = "videobox left=%i right=%i top=%i bottom=%i ! " % \
+                vbox = "videobox left=%i right=%i top=%i bottom=%i ! ffmpegcolorspace ! " % \
                        (-wpx, -wpx, -hpx, -hpx)
             elif width < wmin:
                 px = (wmin - width) / 2
-                vbox = "videobox left=%i right=%i ! " % (-px, -px)
+                vbox = "videobox left=%i right=%i ! ffmpegcolorspace ! " % \
+                       (-px, -px)
             elif height < hmin:
                 px = (hmin - height) / 2
-                vbox = "videobox top=%i bottom=%i ! " % (-px, -px)
-            
-            try:
-                if self.info.videocaps[0].has_key("pixel-aspect-ratio"):
-                    width = int(width * float(self.info.videocaps[0]["pixel-aspect-ratio"]))
-            except KeyError:
-                # The videocaps we are looking for may not even exist, just ignore
-                pass
+                vbox = "videobox top=%i bottom=%i ! ffmpegcolorspace ! " % \
+                       (-px, -px)
             
             # FIXME Odd widths / heights seem to freeze gstreamer
             if width % 2:
@@ -459,14 +492,34 @@ class Transcoder(gobject.GObject):
             if self.options.deinterlace:
                 deint = " ffdeinterlace ! "
             
+            transform = ""
+            if self.preset.vcodec.transform:
+                transform = self.preset.vcodec.transform + " ! "
+            
             sub = ""
             if self.options.subfile:
+                charset = ""
+                if self.options.subfile_charset:
+                    charset = "subtitle-encoding=\"%s\"" % \
+                                                self.options.subfile_charset
+                
                 # Render subtitles onto the video stream
                 sub = "textoverlay font-desc=\"%(font)s\" name=txt ! " % {
                     "font": self.options.font,
                 }
-                cmd += " filesrc location=\"%(subfile)s\" ! subparse ! txt." % {
-                    "subfile": self.options.subfile
+                cmd += " filesrc location=\"%(subfile)s\" ! subparse " \
+                       "%(subfile_charset)s ! txt." % {
+                    "subfile": self.options.subfile,
+                    "subfile_charset": charset,
+                }
+
+            if self.options.ssa is True:             
+                # Render subtitles onto the video stream
+                sub = "textoverlay font-desc=\"%(font)s\" name=txt ! " % {
+                    "font": self.options.font,
+                }
+                cmd += " filesrc location=\"%(infile)s\" ! matroskademux name=demux ! ssaparse ! txt. " % {
+                    "infile": self.infile,
                 }
             
             vmux = premux
@@ -475,9 +528,10 @@ class Transcoder(gobject.GObject):
                     vmux += "video_%d"
             
             cmd += " dmux. ! queue ! ffmpegcolorspace ! videorate !" \
-                   "%s %s videoscale ! %s ! %s%s ! tee " \
+                   "%s %s %s %s videoscale ! %s ! %s%s ! tee " \
                    "name=videotee ! queue ! %s" % \
-                   (deint, sub, self.vcaps.to_string(), vbox, vencoder, vmux)
+                   (deint, vcrop, transform, sub, self.vcaps.to_string(), vbox,
+                    vencoder, vmux)
             
         if self.info.is_audio and self.preset.acodec and \
            self.enc_pass == len(self.preset.vcodec.passes) - 1:
@@ -542,7 +596,8 @@ class Transcoder(gobject.GObject):
                 if premux.startswith("mux"):
                     amux += "audio_%d"
             
-            cmd += " dmux. ! queue ! audioconvert ! audiorate ! " \
+            cmd += " dmux. ! queue ! audioconvert ! " \
+                   "audiorate tolerance=100000000 ! " \
                    "audioresample ! %s ! %s ! %s" % \
                    (self.acaps.to_string(), aencoder, amux)
         
