@@ -7,7 +7,7 @@
     
     License
     -------
-    Copyright 2009 - 2010 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2009 - 2011 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -44,10 +44,14 @@ def get_search_paths():
         @return: A list of paths to search in the order they will be searched
     """
     return [
+        # Current path, useful for development:
         os.getcwd(),
+        # User home directory:
         os.path.expanduser(os.path.join("~", ".arista")),
-        os.path.join(sys.prefix, "share", "arista"),
+        # User-installed:
         os.path.join(sys.prefix, "local", "share", "arista"),
+        # System-installed:
+        os.path.join(sys.prefix, "share", "arista"),
         # The following allows stuff like virtualenv to work!
         os.path.join(os.path.join(os.path.dirname(os.path.dirname(__file__)), "share", "arista")),
     ]
@@ -57,10 +61,10 @@ def get_path(*parts, **kwargs):
         Get a path, searching first in the current directory, then the user's
         home directory, then sys.prefix, then sys.prefix + "local".
         
-            >>> get_path("presets", "computer.xml")
-            '/usr/share/arista/presets/computer.xml'
-            >>> get_path("presets", "my_cool_preset.xml")
-            '/home/dan/.arista/presets/my_cool_preset.xml'
+            >>> get_path("presets", "computer.json")
+            '/usr/share/arista/presets/computer.json'
+            >>> get_path("presets", "my_cool_preset.json")
+            '/home/dan/.arista/presets/my_cool_preset.json'
         
         @type parts: str
         @param parts: The parts of the path to get that you would normally 
@@ -85,6 +89,47 @@ def get_path(*parts, **kwargs):
             "path": path,
         })
 
+def get_write_path(*parts, **kwargs):
+    """
+        Get a path that can be written to. This uses the same logic as get_path
+        above, but instead of checking for the existence of a path it checks
+        to see if the current user has write accces.
+        
+            >>>> get_write_path("presets", "foo.json")
+            '/home/dan/.arista/presets/foo.json'
+        
+        @type parts: str
+        @param parts: The parts of the path to get that you would normally 
+                      send to os.path.join
+        @type default: bool
+        @param default: A default value to return rather than raising IOError
+        @rtype: str
+        @return: The full path to the relative path passed in
+        @raise IOError: The path cannot be written to in any location
+    """
+    path = os.path.join(*parts)
+
+    for search in get_search_paths()[1:]:
+        full = os.path.join(search, path)
+        
+        # Find part of path that exists
+        test = full
+        while not os.path.exists(test):
+            test = os.path.dirname(test)
+        
+        if os.access(test, os.W_OK):
+            if not os.path.exists(os.path.dirname(full)):
+                os.makedirs(os.path.dirname(full))
+                
+            return full
+    else:
+        if "default" in kwargs:
+            return kwargs["default"]
+        
+        raise IOError(_("Can't find %(path)s that can be written to!") % {
+            "path": path,
+        })
+
 def generate_output_path(filename, preset, to_be_created=[],
                          device_name=""):
     """
